import os
from pyspedas.utilities.dailynames import dailynames
from pyspedas.utilities.download import download
from pyspedas.analysis.time_clip import time_clip as tclip
from pytplot import cdf_to_tplot

from .config import CONFIG

def load(trange=['2013-11-5', '2013-11-6'], 
         instrument='dcb',
         datatype='', 
         level='l2', 
         suffix='', 
         get_support_data=False, 
         varformat=None,
         downloadonly=False,
         notplot=False,
         no_update=False,
         time_clip=False):
    """
    This function loads data from the FAST mission; this function is not meant 
    to be called directly; instead, see the wrappers:
        pyspedas.fast.dcb
        pyspedas.fast.acb
        pyspedas.fast.esa
        pyspedas.fast.teams

    """
    file_resolution = 24*3600.

    if instrument == 'dcb':
        pathformat = 'dcf/'+level+'/'+instrument+'/%Y/%m/fast_hr_'+instrument+'_%Y%m%d%H????_v??.cdf'
        file_resolution = 3600.
    if instrument == 'acb':
        pathformat = 'acf/'+level+'/%Y/fa_'+level+'_acf_%Y%m%d_v??.cdf'
    elif instrument == 'esa':
        pathformat = instrument+'/'+level+'/'+datatype+'/%Y/%m/fa_'+instrument+'_'+level+'_'+datatype+'_%Y%m%d??????_*_v??.cdf'
    if instrument == 'teams':
        pathformat = 'teams/'+level+'/%Y/fa_'+level+'_tms_%Y%m%d_v??.cdf'

    # find the full remote path names using the trange
    remote_names = dailynames(file_format=pathformat, trange=trange, res=file_resolution)

    out_files = []

    for remote_file in remote_names:
        files = download(remote_file=remote_file, remote_path=CONFIG['remote_data_dir'], local_path=CONFIG['local_data_dir'], no_download=no_update)
        if files is not None:
            for file in files:
                out_files.append(file)

    out_files = sorted(out_files)

    if downloadonly:
        return out_files

    tvars = cdf_to_tplot(out_files, suffix=suffix, merge=True, get_support_data=get_support_data, varformat=varformat, notplot=notplot)
    
    if notplot:
        return tvars

    if time_clip:
        for new_var in tvars:
            tclip(new_var, trange[0], trange[1], suffix='')

    return tvars





import numpy as np
from pyspedas import time_string, time_double
import os

def dailynames(directory='', trange=None, res=24*3600., hour_res=False, file_format='%Y%m%f', dir_format='', YYYY_MM_DIR=False, prefix='', suffix=''):
    if trange == None:
        print('No trange specified')
        return

    if hour_res == True:
        res = 3600.
        file_format = '%Y%m%d%H'

    if YYYY_MM_DIR:
        dir_format = '%Y/%m/'


    tr = [time_double(trange[0]), time_double(trange[1])]

    # Davin's magic heisted from file_dailynames in IDL
    mmtr = [np.floor(tr[0]/res), np.ceil(tr[1]/res)]

    if mmtr[1]-mmtr[0] < 1:
        n = 1
    else:
        n = int(mmtr[1]-mmtr[0])

    times = [(float(num)+mmtr[0])*res for num in range(n)]
    dates = []
    files = []
        
    for time in times:
        if time_string(time, fmt=file_format) not in dates:
            dates.append(time_string(time, fmt=file_format))
            
    for date in dates:
        files.append(directory + prefix + date + suffix)

    return files


'''

This crib sheet shows how to load and plot HPCA data

'''

from pyspedas import mms_load_hpca
from pytplot import tplot

# load moments data
mms_load_hpca(trange=['2015-10-16/13:05', '2015-10-16/13:10'], data_rate='brst', time_clip=True)

# show H+, O+ and He+ density
tplot(['mms1_hpca_hplus_number_density', 
        'mms1_hpca_oplus_number_density', 
        'mms1_hpca_heplus_number_density'])

# show H+, O+ and He+ temperature
tplot(['mms1_hpca_hplus_scalar_temperature', 
        'mms1_hpca_oplus_scalar_temperature', 
        'mms1_hpca_heplus_scalar_temperature'])

# show H+, O+ and He+ flow velocity
tplot(['mms1_hpca_hplus_ion_bulk_velocity', 
        'mms1_hpca_oplus_ion_bulk_velocity', 
        'mms1_hpca_heplus_ion_bulk_velocity'])

# load the ion data
mms_load_hpca(trange=['2015-10-16/13:05', '2015-10-16/13:07'], datatype='ion', data_rate='brst')

from pyspedas import mms_hpca_calc_anodes
from pyspedas import mms_hpca_spin_sum

# average the flux over the full field of view (0-360)
mms_hpca_calc_anodes(fov=[0, 360], probe='1')

# spin-average to calculate the omni-directional flux
mms_hpca_spin_sum()

# show omni-directional flux for H+, O+ and He+, He++
tplot(['mms1_hpca_hplus_flux_elev_0-360_spin', 
             'mms1_hpca_oplus_flux_elev_0-360_spin', 
             'mms1_hpca_heplus_flux_elev_0-360_spin', 
             'mms1_hpca_heplusplus_flux_elev_0-360_spin'])



# -*- coding: utf-8 -*-
"""
File:
    ex_cdagui.py

Description:
    This is an example of how to use cdagui.
    Since this is a GUI, we only print the instructions here.
    You have to follow these instructions on a python command line,
        for example the Spyder console in Anaconda.

"""


def ex_cdagui():
    # Import pyspedas
    # Run the following command in a python command line:
    #     import pyspedas
    print("--- In a command line, type the following:")
    print("import pyspedas")
    # Open the cdagui by typing
    #     x = pyspedas.cdagui.cdagui()
    print("x = pyspedas.cdagui.cdagui()")
    # In the GUI, do the following:
    # Select 'ARTEMIS', select 'Electric Fields (space)'
    # Click the button '1. Find Datasets' and a list of datasets shoulw appear.
    print("--- The GUI should open.")
    print("--- Select 'ARTEMIS', 'Electric Fields (space)'\
 and click '1. Find Datasets'.")
    # Select 'THB_L2_FIT' and click the button '2. Get File List'
    print("--- Select 'THB_L2_FIT' and click '2. Get File List'.")
    # Select the filename, unselect 'Download Only'
    # and click the button '3. Get Data'
    print("--- Select the filename, unselect 'Download Only'\
 and click '3. Get Data'.")
    # Click 'Exit' to close the GUI
    print("--- Click 'Exit' to close the GUI.")
    # Type the following on the python command line and press Shift+Enter
    #   import pytplot
    #   pytplot.tplot_names()
    # You should get a list of tplot variables loaded.
    print("--- Type the following on the python command line\
 and press Shift+Enter.")
    print("import pytplot")
    print("pytplot.tplot_names()")
    print("--- You should get a list of loaded tplot variables.")
    # Type the following to create a plot:
    #   pytplot.tplot('thb_fgs_dsl')
    print("--- Type the following to create a plot:")
    print("pytplot.tplot('thb_fgs_dsl')")

# Run the example code
# ex_cdagui()



# -*- coding: utf-8 -*-
"""
File:
    time_double.py

Description:
    Transforms datetimes from string to decimal.

Parameters:
    str_time: str/list of str
    If None, then it assumes Now.

Returns:
    float/list of floats
    A UTC decimal time.

Examples:
    time_float()
    time_float(['2018-06-29'])
    time_float(['2017-06-29 23:59:59.1234', '2017-12-01 00:15:21.1234'])
    time_double()

Notes:
    Needs python 3.3+
    Similar to time_double() of IDL SPEDAS
    Test with https://www.epochconverter.com/
"""

from dateutil import parser
from datetime import datetime, timezone


def time_float_one(s_time=None):

    if s_time is None:
        s_time = str(datetime.now())

    if isinstance(s_time, (int, float)):
        return float(s_time)

    in_datetime = parser.parse(s_time)
    float_time = in_datetime.replace(tzinfo=timezone.utc).timestamp()

    return float_time


def time_float(str_time=None):

    if str_time is None:
        return time_float_one()
    else:
        if isinstance(str_time, str):
            return time_float_one(str_time)
        else:
            time_list = list()
            for t in str_time:
                time_list.append(time_float_one(t))
            return time_list


def time_double(str_time=None):
    return time_float(str_time)



# -*- coding: utf-8 -*-
"""
File:
    tnames.py

Description:
    Finds all pytplot names that follow a pattern that may contain a wildcard.

Returns:
    list of str
    A list of tplot names.

Parameters:
    pattern: str/list of str
        List of strings with names and/or patterns.
        If not given, it returns all pytplot names.

Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.
"""

import fnmatch
import pyspedas


def tnames(pattern=None):

    name_list = list()
    all_names = pyspedas.tplot_names()

    if pattern is None:
        name_list.extend(all_names)
    else:
        if isinstance(pattern, str):
            name_list.extend(fnmatch.filter(all_names, pattern))
        else:
            for p in pattern:
                name_list.extend(fnmatch.filter(all_names, p))

    return name_list



'''

This crib sheet shows how to load and plot FEEPS data

'''

from pyspedas import mms_load_feeps, mms_feeps_pad
from pytplot import tplot

# load electron data
mms_load_feeps(trange=['2015-10-16', '2015-10-17'], datatype='electron')

# plot the omni-directional electron intensity
tplot(['mms1_epd_feeps_srvy_l2_electron_intensity_omni', 'mms1_epd_feeps_srvy_l2_electron_intensity_omni_spin'])

# calculate the pitch angle distribution
mms_feeps_pad()

# plot the PAD
tplot('mms1_epd_feeps_srvy_l2_electron_intensity_70-600keV_pad')


'''

This crib sheet shows how to calculate plasma beta using FPI and FGM data

'''

from pyspedas import tinterpol, mms_load_fpi, mms_load_fgm
from pytplot import get_data, store_data, tplot
import numpy as np

mu0 = 1256.0 # nT-m/A
Kb=1.3807*10**(-16.) # cm^2-g-1/s^2-1/K

probe = '1'
trange = ['2015-10-16/11:00', '2015-10-16/14:00']

mms_load_fpi(datatype=['dis-moms', 'des-moms'], trange=trange, probe=probe)
mms_load_fgm(trange=trange, probe=probe)

temp_para_i = 'mms'+probe+'_dis_temppara_fast'
temp_perp_i = 'mms'+probe+'_dis_tempperp_fast'
temp_para_e = 'mms'+probe+'_des_temppara_fast'
temp_perp_e = 'mms'+probe+'_des_tempperp_fast'
number_density_i = 'mms'+probe+'_dis_numberdensity_fast'
number_density_e = 'mms'+probe+'_des_numberdensity_fast'
b_field = 'mms'+probe+'_fgm_b_gsm_srvy_l2'
b_magnitude = 'mms'+probe+'_fgm_b_gsm_srvy_l2_mag'

times, data = get_data(b_field)

store_data(b_magnitude, data={'x': times, 'y': data[:, 3]})

ntimes, ndata = get_data(number_density_i)

n_out_indices = [i[0] for i in np.argwhere(ntimes>=times[0])]

store_data(number_density_i, data={'x': ntimes[n_out_indices], 'y': ndata[n_out_indices]})

tinterpol(b_magnitude, interp_names=[number_density_i], new_names='b_mag_interpolated')

btimes, bdata = get_data('mms1_fgm_b_gsm_srvy_l2_mag-itrp')

ipatimes, i_para_temp = get_data(temp_para_i)
ipetimes, i_perp_temp = get_data(temp_perp_i)
epatimes, e_para_temp = get_data(temp_para_e)
epetimes, e_perp_temp = get_data(temp_perp_e)

nitimes, i_n = get_data(number_density_i)
netimes, e_n = get_data(number_density_e)

# note: 1.0e-8 comes from A-nT/m -> g/(s^2-cm)
Pmag = 1.0e-8*bdata**2/(2.0*mu0)

Te_total=(e_para_temp+2*e_perp_temp)/3.0
Ti_total=(i_para_temp+2*i_perp_temp)/3.0

# note: eV -> K conversion: 11604.505 K/eV
Pplasma = (i_n*11604.505*Ti_total+e_n*11604.505*Te_total)*Kb

Beta = Pplasma/Pmag

store_data('plasma_beta', data={'x': btimes, 'y': Beta})
store_data('magnetic_pressure', data={'x': btimes, 'y': Pmag})
store_data('plasma_pressure', data={'x': btimes, 'y': Pplasma})

tplot(['plasma_beta', 'plasma_pressure', 'magnetic_pressure'])



'''

This crib sheet shows how to load and plot EIS data

'''

from pyspedas import mms_load_eis, mms_eis_pad, tnames
from pytplot import tplot

# load ExTOF data
mms_load_eis(trange=['2015-10-16', '2015-10-17'], datatype='extof')

# plot the H+ flux
tplot(tnames('*_extof_proton_flux_omni'))

# load PHxTOF data
mms_load_eis(trange=['2015-10-16', '2015-10-17'], datatype='phxtof')

# plot the H+ flux
tplot(tnames('*_phxtof_proton_flux_omni'))

# calculate the ExTOF pitch angle distribution
mms_eis_pad(datatype='extof')

tplot(['mms1_epd_eis_extof_56-535keV_proton_flux_omni_pad_spin', 'mms1_epd_eis_extof_56-535keV_proton_flux_omni_pad'])

# calculate the PHxTOF pitch angle distribution
mms_eis_pad(datatype='phxtof', energy=[10, 60])

tplot(['mms1_epd_eis_phxtof_11-57keV_proton_flux_omni_pad_spin', 'mms1_epd_eis_phxtof_11-57keV_proton_flux_omni_pad'])


# -*- coding: utf-8 -*-
# Locations of remote files and local directory for cdf files
# You can change data_dir to a directory of your choice
# If you use Linux, Mac or other unix-like system, use data_dir_unix
data_dir = 'C:\\Datapy\\'
data_dir_unix = '~/Datapy/'
themis_remote = 'http://themis.ssl.berkeley.edu/data/themis'
omni_remote = 'https://cdaweb.gsfc.nasa.gov/istp_public/data/omni'



# -*- coding: utf-8 -*-
"""
File:
    ex_basic.py

Description:
    Basic example to verify that the installation works correctly.
    Downloads THEMIS data and plots it.

"""

import pyspedas
import pytplot


def ex_basic():
    # Delete any existing pytplot variables
    pytplot.del_data()
    # Download THEMIS state data for 2015-12-31
    time_range = ['2015-12-31 00:00:00', '2016-01-01 12:00:00']
    pyspedas.load_data('themis', time_range, ['tha'], 'state', 'l1')
    # Get data into python variables
    alldata = pytplot.get_data("tha_pos")
    time = alldata[0]
    data = alldata[1]
    # Store a new pytplot variable
    pytplot.store_data("tha_position", data={'x': time, 'y': data})
    # Define the y-axis limits
    pytplot.ylim('tha_pos', -23000.0, 81000.0)
    pytplot.ylim('tha_position', -23000.0, 81000.0)
    pytplot.ylim('tha_vel', -8.0, 12.0)
    # Give a title to the plot and labels for the y-axis panels.
    pytplot.tplot_options('title', 'tha position and velocity, 2015-12-31')
    pytplot.options('tha_pos', 'ytitle', 'Position')
    pytplot.options('tha_new_vel', 'ytitle', 'Velocity')
    # Plot position and velocity using the pyqtgraph library (default)
    pytplot.tplot(["tha_pos", "tha_position", "tha_vel"])
    # Plot position and velocity using the bokeh library
    # pytplot.tplot(["tha_pos", "tha_position", "tha_vel"], bokeh=True)

# Run the example code
# ex_basic()



# -*- coding: utf-8 -*-
"""
CDA GUI

Downloads data files from CDAWeb and loads them into pytplot variables.
Requires cdasws, PyQt5.

For cdasws documentation, see:
    https://test.pypi.org/project/cdasws/
    https://cdaweb.sci.gsfc.nasa.gov/WebServices/REST/py/cdasws/index.html

Notes:
    Currently, cdasws is not on the main pypi server.
    pip install -i https://test.pypi.org/simple/ cdasws

    To start the gui from the command line:
        python pyspedas/cdagui/cdagui.py
    To start the gui inside the python environment:
        exec(open('cdagui.py').read())

@author: nikos
"""


import sys
import os
import re
import datetime
import pyspedas
import pytplot
from cdasws import CdasWs
from PyQt5.QtCore import Qt, QDate, QCoreApplication
from PyQt5.QtWidgets import (QApplication, QWidget, QMainWindow,
                             QGridLayout, QPushButton, QListWidget,
                             QGroupBox, QCheckBox, QMessageBox,
                             QVBoxLayout, QLabel, QLineEdit, QListWidgetItem,
                             QFileDialog, QCalendarWidget, QDialog)


def get_default_data_dir():
    """ Returns the default dowload directory for pyspedas.
    """
    data_dir = ''
    prefs = pyspedas.get_spedas_prefs()
    if 'data_dir' in prefs:
        data_dir = prefs['data_dir'] + 'cdaweb'

    return data_dir


def get_instr():
    """ Returns a list of instrument types.
    """
    cdas = CdasWs()
    instr = cdas.get_instrument_types()
    return instr


def get_observatories():
    """ Returns a list of missions.
    """
    cdas = CdasWs()
    observ = cdas.get_observatory_groups()
    return observ


def get_datasets(mission_list, instrument_list):
    """ Returns a list of datasets given the missions and instruments.
    """
    thisdict = {
        "observatoryGroup": mission_list,
        "instrumentType": instrument_list
    }
    cdas = CdasWs()
    dataset = cdas.get_datasets(**thisdict)

    return dataset


def files_to_download(dataset_list, t0, t1):
    """ Find the list of files for a dataset between dates t0 and t1.
        Return a list of url.
    """
    remote_url = []
    max_files = 50
    cdas = CdasWs()
    title = "Server response"

    # Set times to cdas format
    t0 = t0.strip().replace(' ', 'T', 1) + 'Z'
    t1 = t1.strip().replace(' ', 'T', 1) + 'Z'

    # Find dataset list
    for d in dataset_list:
        d0 = d.split(' ')
        if (len(d0) > 1):
            status, result = cdas.get_data_file(d0[0], [], t0, t1)
            if (status == 200 and (result is not None)):
                r = result.get('FileDescription')
                for f in r:
                    remote_url.append(f.get('Name'))

    # Show a message with the result
    msg = ""
    if (status != 200):
        msg = "There was a problem communicating with the SPDF server.\n"
        msg += "Status code:" + str(status)
    else:
        fcount = len(remote_url)
        if (fcount < 1):
            msg = "No files were found with the selected parameters."
        elif (fcount == 1):
            # msg = "Found 1 file.          "
            msg = ""
        else:
            if (fcount > max_files):
                msg = "Found " + str(fcount) + " files.      "
                remote_url = remote_url[0:max_files]
                msg += ("\nOnly a max of " + str(max_files)
                        + " files can be downloaded.")

    if (msg != ""):
        show_my_message(title, msg)

    ''' # For debugging only
    title = 'Debug'
    msg = (t0 + ' --- ' + t1 + ' --- ' + ' = '.join(dlist) +
            ' = '.join(remote_url))
    show_my_message(title, msg)
    '''

    return remote_url


def download_and_load(remote_files, local_dir, download_only=False,
                      varformat=None, get_support_data=False, prefix='',
                      suffix=''):
    """ Download cdf files.
        Load cdf files into pytplot variables.
        TODO: Loading files into pytplot has problems.
    """

    result = []

    """ # For debug only
    msg = local_dir + str(download_only) + " ... " + '==='.join(remote_files)
    title = 'Download Files'
    show_my_message(title, msg)
    print(msg)
    """

    remotehttp = "https://cdaweb.sci.gsfc.nasa.gov/sp_phys/data"
    count = 0
    dcount = 0
    for remotef in remote_files:
        f = remotef.strip().replace(remotehttp, '', 1)
        localf = local_dir + os.path.sep + f

        resp, err, locafile = pyspedas.download_files(remotef, localf)
        count += 1
        if resp:
            print(str(count) + '. File was downloaded. Location: ' + locafile)
            dcount += 1
            result.append(localf)
            if not download_only:
                try:
                    pytplot.cdf_to_tplot(locafile, varformat, get_support_data,
                                         prefix, suffix, False, True)
                except ValueError as err:
                    msg = "cdf_to_tplot could not load " + locafile
                    msg += "\n\n"
                    msg += "Error from pytplot: " + str(err)
                    print(msg)
                    show_my_message("Error!", msg)

        else:
            print(str(count) + '. There was a problem. Could not download \
                  file: ' + remotef)
            print(err)

    print('Downloaded ' + str(dcount) + ' files.')
    print('tplot variables:')
    print(pyspedas.tplot_names())

    return result


def clean_time_str(t):
    """ Removes the time part from datetime variable.
    """
    t0 = re.sub('T.+Z', '', t)
    return t0


def show_my_message(title, msg):
    """ Shows a message.
    """
    alert = QMessageBox()
    alert.setWindowTitle(title)
    alert.setText(msg)
    alert.exec_()


class cdagui(QMainWindow):
    """ Main CDAWeb Window.
    """

    def __init__(self, parent=None):
        super().__init__()
        self.main_widget = GUIWidget(self)
        self.setCentralWidget(self.main_widget)
        self.init_UI()

    def init_UI(self):
        self.statusbar = self.statusBar()
        self.statusbar.showMessage('Status: Ready')
        self.setWindowTitle('CDAWeb Data Downloader')
        self.showMaximized()


class GUIWidget(QWidget):
    """ Main GUI class.
    """

    # Define mission and instrument variables
    mission_list = None
    instrument_list = None
    mission_box1 = None
    instrument_box1 = None
    dataset_box1 = None
    file_box1 = None
    mission_selected = None
    instrument_selected = None
    time_start_box = None
    time_end_box = None
    dir_box = None
    local_dir = None
    button_css = "background-color:#AFEEAF;font-family:Verdana;"
    clear_css = "background-color:#E9967A;font-family:Verdana;"
    title_css = "background-color:#AFEEEE;font-family:Verdana;font-size:14px;"

    def __init__(self, parent):
        super(GUIWidget, self).__init__(parent)
        self.parent = parent
        self.initUI()

    def createMissionGroupBox(self):
        # 1. Missions and instruments group of GUI

        def find_datasets():
            # Fill the dataset list
            # Reads the selections for missions and instruments
            self.dataset_box1.clear()
            self.file_box1.clear()
            self.mission_list = []
            for m in self.mission_box1.selectedIndexes():
                self.mission_list.append(m.data())
            self.mission_selected.setText(str(self.mission_list))
            self.instrument_list = []
            for m in self.instrument_box1.selectedIndexes():
                self.instrument_list.append(m.data())
            self.instrument_selected.setText(str(self.instrument_list))
            allsets = get_datasets(self.mission_list, self.instrument_list)

            allsetslen = len(allsets)

            # Limit dataset results to 100
            datasetmaxshown = 100
            msg = ""
            if allsetslen < 1:
                title = 'Datasets Search'
                msg = ("No datasets found with these parameters: "
                       + str(self.mission_list) + ", "
                       + str(self.instrument_list))
                show_my_message(title, msg)
            elif allsetslen > datasetmaxshown:
                allsets = allsets[0:datasetmaxshown]
                title = 'Datasets Search'
                msg = ("Found " + str(allsetslen) + " datasets with these "
                       + "parameters: "
                       + str(self.mission_list) + ", "
                       + str(self.instrument_list)
                       + ". Only 100 will be shown.")
                show_my_message(title, msg)

            count = 0
            for dataItem in allsets:
                dataSet = dataItem["Id"].strip()
                if len(dataSet) > 1 and dataSet != "(null)":
                    tnow = dataItem["TimeInterval"]
                    t1 = tnow["Start"].strip()
                    t2 = tnow["End"].strip()
                    t1 = clean_time_str(t1)
                    t2 = clean_time_str(t2)

                    dataSet += " (" + t1 + ' to ' + t2 + ")"
                    item = QListWidgetItem(dataSet)
                    self.dataset_box1.addItem(item)
                    count += 1

            # Clear previous selections
            self.instrument_box1.clearSelection()
            self.mission_box1.clearSelection()

            if (msg == ""):
                if (count == 1):
                    msg = "Found 1 dataset."
                else:
                    msg = "Found " + str(count) + " datasets."
            self.parent.statusbar.showMessage('Status: ' + msg)

        # Begin construction of the GUI
        self.missionGroupBox = QGroupBox("Missions and Instruments")
        label1 = QLabel("Mission Groups:")
        label2 = QLabel("Instrument Types:")
        list1 = QListWidget(self)
        list1.setSelectionMode(QListWidget.MultiSelection)
        list1.setMinimumHeight(50)
        list1.setMinimumWidth(400)
        self.mission_box1 = list1
        list2 = QListWidget(self)
        list2.setSelectionMode(QListWidget.MultiSelection)
        list2.setMinimumHeight(50)
        list2.setMinimumWidth(400)
        self.instrument_box1 = list2
        label3 = QLabel("Select one or more Mission Group(s) and one"
                        + " or more Instrument Type(s) and press:")
        button1 = QPushButton("1. Find Datasets")
        button1.setStyleSheet(self.button_css)

        # Create the layout and add GUI elements
        layout = QGridLayout()
        layout.addWidget(label1, 0, 0)
        layout.addWidget(label2, 0, 1)
        layout.addWidget(list1, 1, 0)
        layout.addWidget(list2, 1, 1)
        layout.addWidget(label3, 2, 0, 1, 1)
        layout.addWidget(button1, 2, 1, 1, 1)

        # Fill the mission list
        for dataItem in get_observatories():
            # Contains Name, ShortDescription, LongDescription
            # mission = dataItem["ShortDescription"].strip()
            # mission = dataItem["LongDescription"].strip()
            mission = dataItem["Name"].strip()
            if len(mission) > 1 and mission != "(null)":
                item = QListWidgetItem(mission)
                list1.addItem(item)

        # Fill the instruments list
        for dataItem in get_instr():
            instr = dataItem["Name"].strip()
            if len(instr) > 1 and instr != "(null)":
                item = QListWidgetItem(instr)
                list2.addItem(item)

        # Button1 action
        button1.clicked.connect(find_datasets)

        self.missionGroupBox.setLayout(layout)

    def createDatasetBox(self):
        # 2. Dataset group of GUI

        self.datasetGroupBox = QGroupBox("Datasets")

        label1 = QLabel("Selected Mission Groups:")
        ans1 = QLabel("   ")
        self.mission_selected = ans1
        ans1.setWordWrap(True)
        label2 = QLabel("Selected Instruments:")
        ans2 = QLabel("   ")
        self.instrument_selected = ans2
        ans2.setWordWrap(True)

        listds = QListWidget(self)
        self.dataset_box1 = listds
        listds.setMinimumHeight(50)
        listds.setMinimumWidth(400)
        listds.setSelectionMode(QListWidget.MultiSelection)
        layout = QGridLayout()
        # row, column, rowSpan, columnSpan
        layout.addWidget(label1, 0, 0, 1, 1)
        layout.addWidget(ans1, 0, 1, 1, 5)
        layout.addWidget(label2, 1, 0, 1, 1)
        layout.addWidget(ans2, 1, 1, 1, 5)
        layout.addWidget(listds, 2, 0, 1, 6)

        self.datasetGroupBox.setLayout(layout)

    def createTimeGroupBox(self):
        # 3. Date and time group of GUI

        def get_file_list():
            # Find remote files

            self.file_box1.clear()

            # Find the selected datasets
            dataset_list = []
            msg = ""
            for m in self.dataset_box1.selectedIndexes():
                dataset_list.append(m.data())
            if len(dataset_list) < 1:
                msg = "Please select a dataset."
                title = "No Datasets"
                show_my_message(title, msg)
                self.parent.statusbar.showMessage('Status: ' + msg)
                return

            # Find the the files
            t0 = self.time_start_box.text()
            t1 = self.time_end_box.text()
            remote_file_list = files_to_download(dataset_list, t0, t1)

            count = 0
            for f in remote_file_list:
                item = QListWidgetItem(f)
                self.file_box1.addItem(item)
                count += 1

            if (msg == ""):
                if (count == 1):
                    msg = "Found 1 file."
                else:
                    msg = "Found " + str(count) + " files."
            self.parent.statusbar.showMessage('Status: ' + msg)

        def pick_time(start_or_end):
            dlg = QDialog(self)
            gridc = QVBoxLayout()
            dlg.setLayout(gridc)
            if (start_or_end == "start"):
                title_str = "Start date"
            else:
                title_str = "End date"
            titlelabel = QLabel(title_str)
            gridc.addWidget(titlelabel)

            my_calendar = QCalendarWidget()
            my_calendar.setGridVisible(True)
            my_calendar.move(10, 20)
            gridc.addWidget(my_calendar)

            labeldate = QLabel("")
            gridc.addWidget(labeldate)

            def dial_exit():
                dlg.done(1)

            buttonc = QPushButton("Close")
            buttonc.clicked.connect(dial_exit)
            gridc.addWidget(buttonc)

            def show_date():
                date = my_calendar.selectedDate()
                date_string = date.toString('yyyy-MM-dd')
                if (start_or_end == "start"):
                    self.time_start_box.setText(date_string + " 00:00:01")
                else:
                    self.time_end_box.setText(date_string + " 23:59:59")
                labeldate.setText(date_string)

            my_calendar.clicked[QDate].connect(show_date)

            dlg.setWindowTitle("Calendar")
            dlg.exec_()

        self.timeGroupBox = QGroupBox("Date and Time")

        label1 = QLabel("Start Time:")
        # By default show date 7 days behind to ensure that there is data
        t0 = datetime.datetime.strftime(datetime.datetime.now()
                                        - datetime.timedelta(7), '%Y-%m-%d')
        time1 = QLineEdit(str(t0) + " 00:00:01")
        self.time_start_box = time1
        label2 = QLabel("End Time:")
        time2 = QLineEdit(str(t0) + " 23:59:59")
        self.time_end_box = time2
        label3 = QLabel("Date and time format: YYYY-MM-DD[ HH:MM:SS]")
        button1 = QPushButton("Select")
        button2 = QPushButton("Select")
        button3 = QPushButton("2. Get File List")
        button3.setStyleSheet(self.button_css)

        layout = QGridLayout()
        layout.addWidget(label1, 0, 0)
        layout.addWidget(time1, 0, 1)
        layout.addWidget(button1, 0, 2)
        layout.addWidget(label2, 0, 3)
        layout.addWidget(time2, 0, 4)
        layout.addWidget(button2, 0, 5)
        layout.addWidget(label3, 1, 0, 1, 3)
        layout.addWidget(button3, 1, 3, 1, 3)

        # Calendar pick
        button1.clicked.connect(lambda: pick_time("start"))
        button2.clicked.connect(lambda: pick_time("end"))

        # Button3 action
        button3.clicked.connect(get_file_list)

        self.timeGroupBox.setLayout(layout)

    def createDownloadGroupBox(self):
        # 4. Download group of GUI

        msg2 = ("If checked, then the files will"
                + " only be downloaded. If unchecked, then they will also"
                + " be read and loaded into pytplot variables.")
        check1 = QCheckBox("Download Only")
        check1.setToolTip(msg2)
        check1.setChecked(True)
        buttondown = QPushButton("3. Get Data")
        buttondown.setStyleSheet(self.button_css)

        def clear_all():
            # Clear all boxes
            self.file_box1.clear()
            self.dataset_box1.clear()
            self.instrument_box1.clearSelection()
            self.mission_box1.clearSelection()
            self.instrument_selected.setText('')
            self.mission_selected.setText('')
            self.parent.statusbar.showMessage('Status: Ready')

        def get_data():
            # Download files
            download_only = False
            if check1.isChecked():
                download_only = True

            # Find the selected files
            remote_file_list = []
            for m in self.file_box1.selectedIndexes():
                remote_file_list.append(m.data())
            if len(remote_file_list) < 1:
                title = "No files"
                msg = "Please select some files to download."
                show_my_message(title, msg)
                return
            # Load downloaded files into pytplot
            result = []
            if len(remote_file_list) < 1:
                msg = "No files to download."
                show_my_message(title, msg)
                return

            self.local_dir = self.dir_box.text()

            QApplication.setOverrideCursor(Qt.WaitCursor)
            result = download_and_load(remote_file_list, self.local_dir,
                                       download_only)
            QApplication.restoreOverrideCursor()

            reslen = len(result)
            if reslen > 0:
                msg = "Donwload complete. Number of files: " + str(reslen)
            else:
                msg = "No files were downloaded."

            title = 'Download Files'
            show_my_message(title, msg)
            self.parent.statusbar.showMessage('Status: ' + msg)

        def select_dir():
            file = str(QFileDialog.getExistingDirectory(self,
                                                        "Select Directory"))
            self.local_dir = file
            dir1.setText(self.local_dir)

        def exit_all():
            self.parent.close()
            # The following works on windows but not on linux...
            # app.exit()

        self.dirGroupBox = QGroupBox("Remote Files and Download")

        listf = QListWidget(self)
        self.file_box1 = listf
        listf.setMinimumHeight(50)
        listf.setMinimumWidth(400)
        listf.setSelectionMode(QListWidget.MultiSelection)

        label1 = QLabel("Download Directory:")
        dir1 = QLineEdit()
        self.local_dir = get_default_data_dir()
        dir1.setText(self.local_dir)
        self.dir_box = dir1
        button1 = QPushButton("Change Directory")
        buttonclear = QPushButton("Clear")
        buttonclear.setStyleSheet(self.clear_css)
        buttonexit = QPushButton("Exit")
        buttonexit.setStyleSheet(self.clear_css)

        layout = QGridLayout()
        layout.addWidget(listf, 0, 0, 1, 6)
        layout.addWidget(label1, 1, 0, 1, 1)
        layout.addWidget(dir1, 1, 1, 1, 4)
        layout.addWidget(button1, 1, 5, 1, 1)
        layout.addWidget(check1, 2, 0, 1, 2)
        layout.addWidget(buttondown, 2, 2, 1, 2)
        layout.addWidget(buttonclear, 2, 4, 1, 1)
        layout.addWidget(buttonexit, 2, 5, 1, 1)

        # Button1 action
        button1.clicked.connect(select_dir)
        buttondown.clicked.connect(get_data)
        buttonclear.clicked.connect(clear_all)
        buttonexit.clicked.connect(exit_all)

        self.dirGroupBox.setLayout(layout)

    def initUI(self):
        """ Create GUI
        """

        # Main layout is vertical
        grid = QVBoxLayout()
        self.setLayout(grid)

        # Top label
        label1 = QLabel("Download Data from CDAWeb")
        label1.setStyleSheet(self.title_css)
        label1.setAlignment(Qt.AlignCenter)
        grid.addWidget(label1)

        # 1. Create missions and instruments group
        self.createMissionGroupBox()
        grid.addWidget(self.missionGroupBox)

        # 2. Create dataset group
        self.createDatasetBox()
        grid.addWidget(self.datasetGroupBox)

        # 3. Create datetime group
        self.createTimeGroupBox()
        grid.addWidget(self.timeGroupBox)

        # 4. Create download group
        self.createDownloadGroupBox()
        grid.addWidget(self.dirGroupBox)

        self.showMaximized()


if __name__ == '__main__':
    app = QCoreApplication.instance()
    if app is None:
        app = QApplication(sys.argv)

    cdagui = cdagui()
    app.exec_()



# -*- coding: utf-8 -*-
"""
File:
    ex_line_spectra.py

Description:
    Basic example of plotting both a line and a spectrogram with THEMIS data.
    Downloads THEMIS data and creates a plot.

"""

import pyspedas
import pytplot


def ex_line_spectra():
    # Delete any existing pytplot variables
    pytplot.del_data()
    # Download THEMIS data for 2015-12-31
    pyspedas.load_data('themis', ['2015-12-31'], ['tha'], 'state', 'l1')
    pyspedas.load_data('themis', ['2015-12-31'], ['tha'], 'sst', 'l2')
    # Specify options
    pytplot.ylim('tha_pos', -23000.0, 81000.0)
    pytplot.ylim('tha_psif_en_eflux', 10000.0, 10000000.0)
    pytplot.options('tha_psif_en_eflux', 'colormap', 'viridis')
    pytplot.tplot_options('title', 'tha 2015-12-31')
    # Plot line and spectrogram
    pytplot.tplot(["tha_pos", 'tha_psif_en_eflux'])

# Run the example code
# ex_line_spectra()



# -*- coding: utf-8 -*-
"""
File:
    tinterpol.py

Description:
    Interpolates data specified in 'names' to time values found in 'interp_to'.

Parameters:
    names: str/list of str
        List of variables to interpolate.
    interp_to: str
        String containing the variable containing the time stamps to interpolate to
    method: str
        Interpolation method. Default is ‘linear’.
        Specifies the kind of interpolation as a string (‘linear’, ‘nearest’,
        ‘zero’, ‘slinear’, ‘quadratic’, ‘cubic’, ‘previous’, ‘next’)
        where ‘zero’, ‘slinear’, ‘quadratic’ and ‘cubic’ refer to a spline
        interpolation of zeroth, first, second or third order; ‘previous’ and
        ‘next’ simply return the previous or next value of the point) or
        as an integer specifying the order of the spline interpolator to use.
    newname: str/list of str
        List of new_names for pytplot variables.
        If '', then pytplot variables are replaced.
        If not given, then a suffix is applied.
    suffix:
        A suffix to apply. Default is '-itrp'.

Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.
    Similar to tinterpol in IDL SPEDAS.
"""

import numpy
from pyspedas import tnames
from pytplot import get_data, store_data

def tinterpol(names, interp_to, method=None, newname=None, suffix=None):

    if not isinstance(names, list): names = [names]
    if not isinstance(newname, list): newname = [newname]

    old_names = tnames(names)

    if len(old_names) < 1:
        print('tinterpol error: No pytplot names were provided.')
        return

    if suffix is None:
        suffix = '-itrp'

    if method is None:
        method = 'linear'

    if newname is None:
        n_names = [s + suffix for s in old_names]
    elif newname == '':
        n_names = old_names
    else:
        n_names = newname

    interp_to_data = get_data(interp_to)

    if interp_to_data is None:
        print('Error, tplot variable: ' + interp_to + ' not found.')
        return

    interp_to_times = interp_to_data[0]

    for name_idx, name in enumerate(old_names):
        xdata = get_data(name, xarray=True)
        xdata_interpolated = xdata.interp({'time': interp_to_times}, method=method)

        if 'spec_bins' in xdata.coords:
            store_data(n_names[name_idx], data={'x': interp_to_times, 'y': xdata_interpolated.values, 'v':  xdata_interpolated.coords['spec_bins'].values})
        else:
            store_data(n_names[name_idx], data={'x': interp_to_times, 'y': xdata_interpolated.values})

        print('tinterpol (' + method + ') was applied to: ' + n_names[name_idx])




import numpy as np
from pytplot import get_data, store_data, options
from ...utilities.tnames import tnames

def mms_eis_omni(probe, species='proton', datatype='extof', suffix='', data_units='flux', data_rate='srvy'):
    """
    This function will calculate the omni-directional EIS spectrograms, and is automatically called from mms_load_eis
    
    Parameters:
        probe: str
            probe #, e.g., '4' for MMS4

        species: str
            species for calculation (default: 'proton')

        datatype: str
            'extof' or 'phxtof' (default: 'extof')

        suffix: str
            suffix of the loaded data

        data_units: str
            'flux' or 'cps' (default: 'flux')

        data_rate: str
            instrument data rate, e.g., 'srvy' or 'brst' (default: 'srvy')


    Returns:
        Name of tplot variable created.
    """
    
    probe = str(probe)
    species_str = datatype + '_' + species
    if data_rate == 'brst':
        prefix = 'mms' + probe + '_epd_eis_brst_'
    else: 
        prefix = 'mms' + probe + '_epd_eis_'

    if data_units == 'flux':
        units_label = '1/(cm^2-sr-s-keV)'
    elif data_units == 'cps':
        units_label = '1/s'
    elif data_units == 'counts':
        units_label = 'counts'

    telescopes = tnames(pattern=prefix + species_str + '_*' + data_units + '_t?'+suffix)

    if len(telescopes) == 6:
        time, data, energies = get_data(telescopes[0])
        flux_omni = np.zeros((len(time), len(energies)))
        for t in telescopes:
            time, data, energies = get_data(t)
            flux_omni = flux_omni + data

        store_data(prefix + species_str + '_' + data_units + '_omni' + suffix, data={'x': time, 'y': flux_omni/6., 'v': energies})
        options(prefix + species_str + '_' + data_units + '_omni' + suffix, 'spec', 1)
        options(prefix + species_str + '_' + data_units + '_omni' + suffix, 'ylog', 1)
        options(prefix + species_str + '_' + data_units + '_omni' + suffix, 'zlog', 1)
        options(prefix + species_str + '_' + data_units + '_omni' + suffix, 'ztitle', units_label)
        options(prefix + species_str + '_' + data_units + '_omni' + suffix, 'ytitle', 'MMS' + probe + ' ' + datatype + ' ' + species + ' Energy [keV]')
        options(prefix + species_str + '_' + data_units + '_omni' + suffix, 'yrange', [14, 45])
        options(prefix + species_str + '_' + data_units + '_omni' + suffix, 'Colormap', 'jet')
        return prefix + species_str + '_' + data_units + '_omni' + suffix
    else:
        print('Error, problem finding the telescopes to calculate omni-directional spectrograms')
        return None



# -*- coding: utf-8 -*-
"""
File:
    dates.py

Description:
    Date functions.
"""

import datetime
import dateutil.parser


def validate_date(date_text):
    # Checks if date_text is an acceptable format
    try:
        return dateutil.parser.parse(date_text)
    except ValueError:
        raise ValueError("Incorrect data format, should be yyyy-mm-dd: '"
                         + date_text + "'")


def get_date_list(date_start, date_end):
    """Returns a list of dates between start and end dates"""
    d1 = datetime.date(int(date_start[0:4]), int(date_start[5:7]),
                       int(date_start[8:10]))
    d2 = datetime.date(int(date_end[0:4]), int(date_end[5:7]),
                       int(date_end[8:10]))

    delta = d2 - d1
    ans_list = []

    for i in range(delta.days + 1):
        ans_list.append(str(d1 + datetime.timedelta(days=i)))

    return ans_list


def get_dates(dates):
    """Returns a list of dates
       date format: yyyy-mm-dd
       input can be a single date or a list of two (start and end date)
    """
    ans_list = []
    if not isinstance(dates, (list, tuple)):
        dates = [dates]

    if len(dates) == 1:
        try:
            validate_date(dates[0])
            ans_list = dates
        except ValueError as e:
            print(e)
            ans_list = []
    elif len(dates) == 2:
        try:
            validate_date(dates[0])
            validate_date(dates[1])
            ans_list = get_date_list(dates[0], dates[1])
        except ValueError as e:
            print(e)
            ans_list = []

    return ans_list




from .load import load

def mfe(trange=['2003-10-28', '2003-10-29'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Magnetic Field Experiment
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='mfe', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def efi(trange=['2003-10-28', '2003-10-29'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Electric Fields Instrument
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='efi', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def pwi(trange=['1997-01-03', '1997-01-04'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Plasma Wave Instrument
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='pwi', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def hydra(trange=['2003-10-28', '2003-10-29'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Hot Plasma Analyzer Experiment
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='hydra', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def tide(trange=['1997-01-03', '1997-01-04'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Thermal Ion Dynamics Experiment / Plasma Source Investigation
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='tide', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def timas(trange=['1997-01-03', '1997-01-04'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Toroidal Imaging Mass Angle Spectrograph
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='timas', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def cammice(trange=['2003-10-28', '2003-10-29'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Charge and Mass Magnetospheric Ion Composition Experiment
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='cammice', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def ceppad(trange=['2003-10-28', '2003-10-29'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Comprehensive Energetic Particle-Pitch Angle Distribution
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='ceppad', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def uvi(trange=['2003-10-28', '2003-10-29'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Ultraviolet Imager
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='uvi', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def vis(trange=['2003-10-28', '2003-10-29'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Visible Imaging System
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='vis', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def pixie(trange=['1997-01-03', '1997-01-04'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads data from the Polar Ionospheric X-ray Imaging Experiment
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='pixie', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)

def orbit(trange=['2003-10-28', '2003-10-29'],
        datatype='k0',
        suffix='',  
        get_support_data=False, 
        varformat=None,
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False):
    """
    This function loads Polar orbit data
    
    Parameters:
        trange : list of str
            time range of interest [starttime, endtime] with the format 
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day 
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        datatype: str
            Data type; Valid options:

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a 
            "VAR_TYPE" attribute of "data".

        varformat: str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into 
            tplot variables

    Returns:
        List of tplot variables created.

    """
    return load(instrument='spha', trange=trange, datatype=datatype, suffix=suffix, get_support_data=get_support_data, varformat=varformat, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)




import numpy as np

def dpwrspc(time, quantity, nboxpoints=256, nshiftpoints=128, binsize=3, nohanning=False, noline=False, notperhz=False, notmvariance=False):

    if nohanning is False:
        window = np.hanning(nboxpoints)

    # remove NaNs from the data
    where_finite = np.where(np.isnan(quantity)==False)

    quantity2process = quantity[where_finite[0]]
    times2process = time[where_finite[0]]
    nboxpnts = nboxpoints
    nshiftpnts = nshiftpoints


    totalpoints = len(times2process)
    nspectra = int((totalpoints-nboxpnts/2.)/nshiftpnts)

    # test nspectra, if the value of nshiftpnts is much smaller than
    # nboxpnts/2 strange things happen

    nbegin = np.array([nshiftpnts*i for i in range(nspectra)])
    nend = nbegin + nboxpnts

    okspec = np.where(nend <= totalpoints-1)

    if len(okspec[0]) <= 0:
        print('Not enough points for a calculation')
        return

    tdps = np.zeros(nspectra)
    nfreqs = int(int(nboxpnts/2)/binsize)

    if nfreqs <= 1:
        print('Not enough frequencies for a calculation')
        return

    dps = np.zeros([nspectra, nfreqs])
    fdps = np.zeros([nspectra, nfreqs])

    for nthspectrum in range(0, nspectra):
        nbegin = int(nthspectrum*nshiftpnts)
        nend = nbegin + nboxpnts

        if nend <= totalpoints-1:
            t = times2process[nbegin:nend]
            t0 = t[0]
            t = t - t0
            x = quantity2process[nbegin:nend]

            # Use center time
            tdps[nthspectrum] = (times2process[nbegin]+times2process[nend])/2.0

            if noline is False:
                coef = np.polyfit(t, x, 1)
                poly1d_fn = np.poly1d(coef)
                line = poly1d_fn(t)
                x = x - line


            if nohanning is False:
                x = x*window

            bign = nboxpnts

            if bign % 2 != 0:
                print('needs an even number of data points, dropping last point...')
                t = t[0:bign-1]
                x = x[0:bign-1]
                bign = bign - 1

            n_tm = len(t)

            # time variance can break power spectrum, this keyword skips over those gaps
            if notmvariance and n_tm > 1:
                if tm_sensitivity:
                    tmsn = tm_sensitivity
                else:
                    tmsn = 100.0

                tdiff = t[1:n_tm]-t[0:n_tm-1]
                med_diff = np.median(tdiff)

                idx = np.where(np.abs(tdiff/med_diff-1) > 1.0/tmsn)

                if len(idx[0]) > 0:
                    dps[nthspectrum, :] = float('nan')
                    fdps[nthspectrum, :] = float('nan')

                    continue

            # following Numerical recipes in Fortran, p. 421, sort of... (actually following the IDL implementation)
            k = np.array(range(int(bign/2)+1))
            tres = np.median(t[1:len(t)] - t[0:len(t)-1])
            fk = k/(bign*tres)

            xs2 = np.abs(np.fft.fft(x))**2

            pwr = np.zeros(int(bign/2+1))
            pwr[0] = xs2[0]/bign**2
            pwr[1:int(bign/2)] = (xs2[1:int(bign/2)] + xs2[bign-(1+np.array(range(int(bign/2-1))))])/bign**2
            pwr[int(bign/2)] = xs2[int(bign/2)]/bign**2

            if nohanning is False:
                wss = bign*np.sum(window**2)
                pwr = bign**2*pwr/wss

            dfreq = binsize*(fk[1]-fk[0])

            npwr = len(pwr)-1
            nfinal = int(npwr/binsize)
            iarray = np.array(range(nfinal))
            power = np.zeros(nfinal)

            # Note: zeroth point includes zero freq. power.
            freqcenter = (fk[iarray*binsize+1]+fk[iarray*binsize+binsize])/2.

            for i in range(binsize):
                power = power+pwr[iarray*binsize+i+1]

            if notperhz is False:
                power=power/dfreq

            dps[nthspectrum, :] = power
            fdps[nthspectrum, :] = freqcenter

    return (tdps, fdps, dps)




#!/usr/bin/python
# -*- coding: utf-8 -*-

import os

CONFIG = {'local_data_dir': 'pydata',
         #'local_data_dir': '/Users/eric/data/mms', # example of setting your local data directory on macOS
         #'local_data_dir': 'c:\users\eric\data\mms', # and Windows
          'mirror_data_dir': None, # e.g., '/Volumes/data_network/data/mms'
          'debug_mode': False,
          'download_only': False,
          'no_download': False}

# override local data directory with environment variables
if os.environ.get('ROOT_DATA_DIR'):
	CONFIG['local_data_dir'] = os.sep.join([os.environ['ROOT_DATA_DIR'], 'mms'])

if os.environ.get('MMS_DATA_DIR'):
	CONFIG['local_data_dir'] = os.environ['MMS_DATA_DIR']


import os
from pyspedas.utilities.dailynames import dailynames
from pyspedas.utilities.download import download
from pyspedas.analysis.time_clip import time_clip as tclip
from pytplot import cdf_to_tplot

from .config import CONFIG

def load(trange=['2018-10-16', '2018-10-17'], 
         instrument='mag', 
         datatype='h0', 
         suffix='', 
         get_support_data=False, 
         varformat=None,
         downloadonly=False,
         notplot=False,
         no_update=False,
         time_clip=False):
    """
    This function loads DSCOVR data into tplot variables; this function is not 
    meant to be called directly; instead, see the wrappers: 
        dscovr.mag: Fluxgate Magnetometer data
        dscovr.fc: Faraday Cup data
        dscovr.orb: Ephemeris data
        dscovr.att: Attitude data
        dscovr.all: Load all data

    """

    remote_path = datatype + '/' + instrument + '/%Y/'

    if instrument == 'mag':
        if datatype == 'h0':
            pathformat = remote_path + 'dscovr_h0_mag_%Y%m%d_v??.cdf'
    elif instrument == 'faraday_cup':
        if datatype == 'h1':
            pathformat = remote_path + 'dscovr_h1_fc_%Y%m%d_v??.cdf'
    elif instrument == 'pre_or':
        if datatype == 'orbit':
            pathformat = remote_path + 'dscovr_orbit_pre_%Y%m%d_v??.cdf'
    elif instrument == 'def_at':
        if datatype == 'orbit':
            pathformat = remote_path + 'dscovr_at_def_%Y%m%d_v??.cdf'

    # find the full remote path names using the trange
    remote_names = dailynames(file_format=pathformat, trange=trange)

    out_files = []

    for remote_file in remote_names:
        files = download(remote_file=remote_file, remote_path=CONFIG['remote_data_dir'], local_path=CONFIG['local_data_dir'], no_download=no_update)
        if files is not None:
            for file in files:
                out_files.append(file)

    out_files = sorted(out_files)

    if downloadonly:
        return out_files

    tvars = cdf_to_tplot(out_files, suffix=suffix, merge=True, get_support_data=get_support_data, varformat=varformat, notplot=notplot)

    if notplot:
        return tvars
        
    if time_clip:
        for new_var in tvars:
            tclip(new_var, trange[0], trange[1], suffix='')

    return tvars



'''

This crib sheet shows how to load and plot DSP data

'''

from pyspedas import mms_load_dsp
from pytplot import tplot, options

# load the electric spectral density
mms_load_dsp(data_rate='fast', probe=1, datatype='epsd', level='l2')

options('mms1_dsp_epsd_omni', 'spec', True)
options('mms1_dsp_epsd_omni', 'zlog', True)
options('mms1_dsp_epsd_omni', 'ylog', True)
options('mms1_dsp_epsd_omni', 'Colormap', 'jet')
options('mms1_dsp_epsd_omni', 'zrange', [1e-14, 1e-4])
options('mms1_dsp_epsd_omni', 'yrange', [30, 1e5])

tplot(['mms1_dsp_epsd_omni'])

# load the magnetic spectral density
mms_load_dsp(data_rate='fast', probe=1, datatype='bpsd', level='l2')

options('mms1_dsp_bpsd_omni_fast_l2', 'spec', True)
options('mms1_dsp_bpsd_omni_fast_l2', 'zlog', True)
options('mms1_dsp_bpsd_omni_fast_l2', 'ylog', True)
options('mms1_dsp_bpsd_omni_fast_l2', 'Colormap', 'jet')
options('mms1_dsp_bpsd_omni_fast_l2', 'zrange', [1e-14, 10])
options('mms1_dsp_bpsd_omni_fast_l2', 'yrange', [10, 1e4])

# show the omni-directional SCM spectral density for MMS-1
tplot('mms1_dsp_bpsd_omni_fast_l2')



# -*- coding: utf-8 -*-
"""
File:
    themis_helpers.py

Description:
    Helper functions for themis.
"""


def get_probes(probes):
    """Returns a list of probes"""
    probe_list = ['a', 'b', 'c', 'd', 'e']
    thprobe_list = ['tha', 'thb', 'thc', 'thd', 'the']
    ans_list = []
    if not isinstance(probes, (list, tuple)):
        probes = [probes]

    for p in probes:
        p = p.lower()
        if p == '*':
            ans_list = thprobe_list
            break
        if p in probe_list:
            ans_list.append('th' + p)
        elif p in thprobe_list:
            ans_list.append(p)

    return ans_list


def get_instruments(instruments, level):
    """Returns a list of themis instruments for L2 data"""
    if level == 'l1':
        instr_list = ['bau', 'eff', 'efp', 'efw', 'esa', 'fbk', 'fff_16',
                      'fff_32', 'fff_64', 'ffp_16', 'ffp_32', 'ffp_64',
                      'ffw_16', 'ffw_32', 'ffw_64', 'fgm', 'fit', 'hsk',
                      'mom', 'scf', 'scm', 'scmode', 'scp', 'scw', 'spin',
                      'sst', 'state', 'trg', 'vaf', 'vap', 'vaw']
    elif level == 'l2_mag':
        instr_list = instruments
    else:
        instr_list = ['efi', 'esa', 'fbk', 'fft', 'fgm', 'fit', 'gmom',
                      'mom', 'scm', 'sst']
    ans_list = []
    if not isinstance(instruments, (list, tuple)):
        instruments = [instruments]

    for p in instruments:
        p = p.lower()
        if p == '*':
            ans_list = instr_list
            break
        if p in instr_list:
            ans_list.append(p)

    return ans_list



#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest
import numpy as np

from ...mms import mms_load_fgm, mms_load_scm, mms_load_fpi, mms_load_hpca, mms_load_eis, mms_load_feeps, mms_load_edp, mms_load_edi, mms_load_aspoc, mms_load_dsp
from ...utilities.data_exists import data_exists
from ..hpca.mms_hpca_calc_anodes import mms_hpca_calc_anodes
from ..hpca.mms_hpca_spin_sum import mms_hpca_spin_sum

from pytplot import get_data, del_data

############### DSP ############### 
class DSPLoadTestCases(unittest.TestCase):
    def test_load_default_data(self):
        data = mms_load_dsp(trange=['2015-08-01','2015-09-01'], datatype='swd', level='l2', data_rate='fast')
        self.assertTrue(data_exists('mms1_dsp_swd_E12_Counts'))

    def test_load_epsd_bpsd_data(self):
        data = mms_load_dsp(trange=['2015-08-01','2015-08-02'], datatype=['epsd', 'bpsd'], level='l2', data_rate='fast')
        self.assertTrue(data_exists('mms1_dsp_epsd_omni'))
        self.assertTrue(data_exists('mms1_dsp_bpsd_omni'))

    def test_load_epsd_suffix(self):
        data = mms_load_dsp(trange=['2015-08-01','2015-08-02'], datatype='epsd', level='l2', data_rate='fast', suffix='_test')
        self.assertTrue(data_exists('mms1_dsp_epsd_omni_test'))

############### FEEPS ############### 
class FEEPSLoadTestCases(unittest.TestCase):
    def test_load_default_data(self):
        data = mms_load_feeps(trange=['2015-10-16', '2015-10-16/01:00'])
        self.assertTrue(data_exists('mms1_epd_feeps_srvy_l2_electron_intensity_omni'))
        self.assertTrue(data_exists('mms1_epd_feeps_srvy_l2_electron_intensity_omni_spin'))

    def test_load_suffix(self):
        data = mms_load_feeps(trange=['2015-10-16', '2015-10-16/01:00'], suffix='_test')
        self.assertTrue(data_exists('mms1_epd_feeps_srvy_l2_electron_intensity_omni_test'))
        self.assertTrue(data_exists('mms1_epd_feeps_srvy_l2_electron_intensity_omni_spin_test'))

    def test_load_brst_data(self):
        del_data('*')
        data = mms_load_feeps(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst')
        self.assertTrue(data_exists('mms1_epd_feeps_brst_l2_electron_intensity_omni'))
        self.assertTrue(data_exists('mms1_epd_feeps_brst_l2_electron_intensity_omni_spin'))

    def test_load_brst_multi_probe(self):
        del_data('*')
        data = mms_load_feeps(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst', probe=[1, 2, 3, 4])
        self.assertTrue(data_exists('mms1_epd_feeps_brst_l2_electron_intensity_omni'))
        self.assertTrue(data_exists('mms1_epd_feeps_brst_l2_electron_intensity_omni_spin'))
        self.assertTrue(data_exists('mms2_epd_feeps_brst_l2_electron_intensity_omni'))
        self.assertTrue(data_exists('mms2_epd_feeps_brst_l2_electron_intensity_omni_spin'))
        self.assertTrue(data_exists('mms3_epd_feeps_brst_l2_electron_intensity_omni'))
        self.assertTrue(data_exists('mms3_epd_feeps_brst_l2_electron_intensity_omni_spin'))
        self.assertTrue(data_exists('mms4_epd_feeps_brst_l2_electron_intensity_omni'))
        self.assertTrue(data_exists('mms4_epd_feeps_brst_l2_electron_intensity_omni_spin'))

############### EIS ############### 
class EISLoadTestCases(unittest.TestCase):
    def test_load_phxtof_data(self):
        data = mms_load_eis(trange=['2015-10-16', '2015-10-16/01:00'], datatype='phxtof')
        self.assertTrue(data_exists('mms1_epd_eis_phxtof_proton_flux_omni'))

    def test_load_extof_data(self):
        data = mms_load_eis(trange=['2015-10-16', '2015-10-16/01:00'], datatype='extof')
        self.assertTrue(data_exists('mms1_epd_eis_extof_proton_flux_omni'))

    def test_load_extof_suffix(self):
        data = mms_load_eis(trange=['2015-10-16', '2015-10-16/01:00'], datatype='extof', suffix='_test')
        self.assertTrue(data_exists('mms1_epd_eis_extof_proton_flux_omni_test'))
        self.assertTrue(data_exists('mms1_epd_eis_extof_proton_flux_omni_test_spin'))

############### FPI ############### 
class FPILoadTestCases(unittest.TestCase):
    def test_load_default_data(self):
        data = mms_load_fpi(trange=['2015-10-16/14:00', '2015-10-16/15:00'])
        self.assertTrue(data_exists('mms1_dis_energyspectr_omni_fast'))

    def test_load_small_brst_interval(self):
        data = mms_load_fpi(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst', datatype='dis-moms')
        self.assertTrue(data_exists('mms1_dis_energyspectr_omni_brst'))

    def test_center_fast_ion_data_notplot(self):
        data = mms_load_fpi(trange=['2015-10-16/14:00', '2015-10-16/15:00'], notplot=True)
        centered = mms_load_fpi(trange=['2015-10-16/14:00', '2015-10-16/15:00'], center_measurement=True, suffix='_centered', notplot=True)

        self.assertTrue(np.round(centered['mms1_dis_bulkv_gse_fast_centered']['x'][0]-data['mms1_dis_bulkv_gse_fast']['x'][0], decimals=3) == 2.25)

    def test_center_fast_ion_data(self):
        data = mms_load_fpi(trange=['2015-10-16/14:00', '2015-10-16/15:00'])
        centered = mms_load_fpi(trange=['2015-10-16/14:00', '2015-10-16/15:00'], center_measurement=True, suffix='_centered')
        
        t, d = get_data('mms1_dis_bulkv_gse_fast')
        c, d = get_data('mms1_dis_bulkv_gse_fast_centered')
        self.assertTrue(np.round(c[0]-t[0], decimals=3) == 2.25)

    def test_center_fast_electron_data(self):
        data = mms_load_fpi(trange=['2015-10-16/14:00', '2015-10-16/15:00'])
        centered = mms_load_fpi(trange=['2015-10-16/14:00', '2015-10-16/15:00'], center_measurement=True, suffix='_centered')
        
        t, d = get_data('mms1_des_bulkv_gse_fast')
        c, d = get_data('mms1_des_bulkv_gse_fast_centered')
        self.assertTrue(np.round(c[0]-t[0], decimals=3) == 2.25)

    def test_center_brst_ion_data(self):
        data = mms_load_fpi(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst')
        centered = mms_load_fpi(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst', center_measurement=True, suffix='_centered')
        
        t, d = get_data('mms1_dis_bulkv_gse_brst')
        c, d = get_data('mms1_dis_bulkv_gse_brst_centered')
        self.assertTrue(np.round(c[0]-t[0], decimals=3) == 0.075)

    def test_center_brst_electron_data(self):
        data = mms_load_fpi(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst')
        centered = mms_load_fpi(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst', center_measurement=True, suffix='_centered')
        
        t, d = get_data('mms1_des_bulkv_gse_brst')
        c, d = get_data('mms1_des_bulkv_gse_brst_centered')
        self.assertTrue(np.round(c[0]-t[0], decimals=3) == 0.015)

############### HPCA ############### 
class HPCALoadTestCases(unittest.TestCase):
    def test_load_default_data(self):
        data = mms_load_hpca(trange=['2015-10-16', '2015-10-16/01:00'])
        self.assertTrue(data_exists('mms1_hpca_hplus_number_density'))

    def test_load_ion_data(self):
        data = mms_load_hpca(trange=['2015-10-16/5:00', '2015-10-16/6:00'], datatype='ion')
        self.assertTrue(data_exists('mms1_hpca_hplus_flux'))

    def test_load_ion_omni(self):
        del_data('*')
        data = mms_load_hpca(trange=['2016-10-16/5:00', '2016-10-16/6:00'], datatype='ion')
        mms_hpca_calc_anodes(fov=[0, 360], probe='1')
        mms_hpca_spin_sum()
        self.assertTrue(data_exists('mms1_hpca_hplus_flux_elev_0-360_spin'))
        self.assertTrue(data_exists('mms1_hpca_heplus_flux_elev_0-360_spin'))
        self.assertTrue(data_exists('mms1_hpca_heplusplus_flux_elev_0-360_spin'))
        self.assertTrue(data_exists('mms1_hpca_oplus_flux_elev_0-360_spin'))

    def test_center_fast_moments_data(self):
        data = mms_load_hpca(trange=['2015-10-16/14:00', '2015-10-16/15:00'])
        centered = mms_load_hpca(trange=['2015-10-16/14:00', '2015-10-16/15:00'], center_measurement=True, suffix='_centered')
        
        t, d = get_data('mms1_hpca_hplus_ion_bulk_velocity')
        c, d = get_data('mms1_hpca_hplus_ion_bulk_velocity_centered')
        self.assertTrue(np.round(c[0]-t[0], decimals=3) == 5.0)

    def test_center_brst_moments_data(self):
        data = mms_load_hpca(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst')
        centered = mms_load_hpca(trange=['2015-10-16/13:06', '2015-10-16/13:07'], data_rate='brst', center_measurement=True, suffix='_centered')
        
        t, d = get_data('mms1_hpca_hplus_ion_bulk_velocity')
        c, d = get_data('mms1_hpca_hplus_ion_bulk_velocity_centered')
        self.assertTrue(np.round(c[0]-t[0], decimals=3) == 5.0)

############### EDI ############### 
class EDILoadTestCases(unittest.TestCase):
    def test_load_default_data(self):
        data = mms_load_edi(trange=['2016-10-17/13:00', '2016-10-17/14:00'])
        self.assertTrue(data_exists('mms1_edi_e_gse_srvy_l2'))

    def test_load_suffix(self):
        data = mms_load_edi(trange=['2016-10-17/13:00', '2016-10-17/14:00'], suffix='_test')
        self.assertTrue(data_exists('mms1_edi_e_gse_srvy_l2_test'))

############### ASPOC ############### 
class ASPOCLoadTestCases(unittest.TestCase):
    def test_load_default_data(self):
        data = mms_load_aspoc(trange=['2015-10-16', '2015-10-16/01:00'])
        self.assertTrue(data_exists('mms1_aspoc_ionc'))

    def test_load_suffix(self):
        data = mms_load_aspoc(trange=['2015-10-16', '2015-10-16/01:00'], suffix='_test')
        self.assertTrue(data_exists('mms1_aspoc_ionc_test'))

############### EDP ############### 
class EDPLoadTestCases(unittest.TestCase):
    def test_load_default_data(self):
        data = mms_load_edp(trange=['2015-10-16', '2015-10-16/01:00'])
        self.assertTrue(data_exists('mms1_edp_dce_gse_fast_l2'))

    def test_load_suffix(self):
        data = mms_load_edp(trange=['2015-10-16', '2015-10-16/01:00'], suffix='_test')
        self.assertTrue(data_exists('mms1_edp_dce_gse_fast_l2'))

    def test_load_multiple_sc(self):
        data = mms_load_edp(probe=['1', '2', '3', '4'], trange=['2015-10-16', '2015-10-16/01:00'])
        self.assertTrue(data_exists('mms1_edp_dce_gse_fast_l2'))
        self.assertTrue(data_exists('mms2_edp_dce_gse_fast_l2'))
        self.assertTrue(data_exists('mms3_edp_dce_gse_fast_l2'))
        self.assertTrue(data_exists('mms4_edp_dce_gse_fast_l2'))

    def test_load_brst_data(self):
        data = mms_load_edp(data_rate='brst', trange=['2015-10-16/13:06', '2015-10-16/13:10'])
        self.assertTrue(data_exists('mms1_edp_dce_gse_brst_l2'))

############### FGM ############### 
class FGMLoadTestCases(unittest.TestCase):
    def test_load_default_data(self):
        data = mms_load_fgm(trange=['2015-10-16', '2015-10-16/01:00'])
        self.assertTrue(data_exists('mms1_fgm_b_gse_srvy_l2'))

    def test_load_suffix(self):
        data = mms_load_fgm(trange=['2015-10-16', '2015-10-16/01:00'], suffix='_test')
        self.assertTrue(data_exists('mms1_fgm_b_gse_srvy_l2_test'))

    def test_load_multiple_sc(self):
        data = mms_load_fgm(probe=['1', '2', '3', '4'], trange=['2015-10-16', '2015-10-16/01:00'])
        self.assertTrue(data_exists('mms1_fgm_b_gse_srvy_l2'))
        self.assertTrue(data_exists('mms2_fgm_b_gse_srvy_l2'))
        self.assertTrue(data_exists('mms3_fgm_b_gse_srvy_l2'))
        self.assertTrue(data_exists('mms4_fgm_b_gse_srvy_l2'))

    def test_load_brst_data(self):
        data = mms_load_fgm(data_rate='brst', trange=['2015-10-16/13:06', '2015-10-16/13:10'])
        self.assertTrue(data_exists('mms1_fgm_b_gse_brst_l2'))

    def test_load_int_probes(self):
        data = mms_load_fgm(probe=[1, 2, 3, 4], trange=['2015-10-16', '2015-10-16/01:00'])
        self.assertTrue(data_exists('mms1_fgm_b_gse_srvy_l2'))

############### SCM ############### 
# class SCMLoadTestCases(unittest.TestCase):
#     def test_load_default_data(self):
#         data = mms_load_scm(trange=['2015-10-16', '2015-10-16/01:00'])
#         self.assertTrue(data_exists('mms1_scm_acb_gse_scsrvy_srvy_l2'))

#     def test_load_suffix(self):
#         data = mms_load_scm(trange=['2015-10-16', '2015-10-16/01:00'], suffix='_test')
#         self.assertTrue(data_exists('mms1_scm_acb_gse_scsrvy_srvy_l2_test'))

#     def test_load_multiple_sc(self):
#         data = mms_load_scm(probe=['1', '2', '3', '4'], trange=['2017-12-15', '2017-12-16'])
#         # self.assertTrue(data_exists('mms1_scm_acb_gse_scsrvy_srvy_l2'))
#         # self.assertTrue(data_exists('mms2_scm_acb_gse_scsrvy_srvy_l2'))
#         self.assertTrue(data_exists('mms3_scm_acb_gse_scsrvy_srvy_l2'))
#         self.assertTrue(data_exists('mms4_scm_acb_gse_scsrvy_srvy_l2'))

#     def test_load_brst_data(self):
#         data = mms_load_scm(data_rate='brst', trange=['2015-10-16/13:06', '2015-10-16/13:10'], datatype='scb')
#         self.assertTrue(data_exists('mms1_scm_acb_gse_scb_brst_l2'))

if __name__ == '__main__':
    unittest.main()